@@ -273,21 +273,31 @@ class Quote(models.Model):
 
 class Project(models.Model):
 
+    ACTIVE = "AC"
+    INACTIVE = "IN"
+    ARCHIVED = "AR"
+    STATUS_CHOICES = [
+        (ACTIVE, "Active"),
+        (INACTIVE, "Inactive"),
+        (ARCHIVED, "Archived"),
+    ]
+
     id = models.BigAutoField(primary_key=True)
     user = models.ForeignKey(settings.AUTH_USER_MODEL, on_delete=models.CASCADE)
     date_creation = models.DateTimeField(auto_now_add=True, auto_now=False)
     date_modification = models.DateTimeField(auto_now_add=False, auto_now=True)
-    title = models.CharField(max_length=255)
-    category = models.CharField(max_length=255, default="general")
-    limit_date = models.DateField(blank=True, null=True)
-    progress_min = models.PositiveIntegerField(blank=True, null=True)
-    progress_max = models.PositiveIntegerField(blank=True, null=True)
-    progress_current = models.PositiveIntegerField(blank=True, null=True)
-    description = models.TextField(blank=True, null=True)
+    title = models.CharField(max_length=255, blank=True, null=True)
+    category = models.CharField(max_length=255, default="general")        # Deprecated
+    limit_date = models.DateField(blank=True, null=True)                  # Deprecated
+    progress_min = models.PositiveIntegerField(blank=True, null=True)     # Deprecated
+    progress_max = models.PositiveIntegerField(blank=True, null=True)     # Deprecated
+    progress_current = models.PositiveIntegerField(blank=True, null=True) # Deprecated
+    description = models.TextField(blank=True, null=True)                 # Deprecated
     checklist = models.TextField(blank=True, null=True)
-    rank = models.FloatField()
+    rank = models.FloatField()                                            # Deprecated
     note = models.ForeignKey("Note", on_delete=models.SET_NULL, null=True, blank=True)
-    archived = models.BooleanField(default=False)
+    archived = models.BooleanField(default=False)                         # Deprecated
+    status = models.CharField(max_length=2, choices=STATUS_CHOICES, default=ACTIVE)
 
     class Meta:
 
@@ -297,7 +307,48 @@ class Project(models.Model):
         return f"{ self.user} - { self.id }. { self.title }"
 
     def get_absolute_url(self):
-        return reverse("orgapy:projects") + f"#project-{self.id}"
+        return reverse("orgapy:project", kwargs={"object_id": self.id})
+    
+    @property
+    def items_count(self) -> int:
+        if self.checklist is None:
+            return 0
+        return len(re.findall(r"^\[[ x]\]", self.checklist, re.MULTILINE))
+    
+    @property
+    def completed_items_count(self) -> int:
+        if self.checklist is None:
+            return 0
+        return len(re.findall(r"^\[x\]", self.checklist, re.MULTILINE))
+
+    @property
+    def all_completed(self) -> bool:
+        return self.completed_items_count == self.items_count
+
+    @property
+    def reference(self) -> str:
+        if self.note is None and self.title is not None:
+            return self.title
+        if self.note is not None and self.title is None:
+            return self.note.title
+        if self.note is not None and self.title is not None:
+            return f"{self.note.title} - {self.title}"
+        return "Untitled"
+    
+    def to_json_dict(self) -> dict:
+        return {
+            "id": self.id,
+            "creation": self.date_creation.timestamp(),
+            "modification": self.date_modification.timestamp(),
+            "title": self.title,
+            "checklist": self.checklist if self.checklist else None,
+            "note": None if self.note is None else {
+                "id": self.note.id,
+                "title": self.note.title,
+                "url": self.note.get_absolute_url(),
+            },
+            "status": self.status,
+        }
 
 
 class Calendar(models.Model):
