@@ -6,7 +6,7 @@ import dateutil.relativedelta
 from django.contrib.auth.decorators import permission_required
 from django.contrib.auth.models import AnonymousUser
 from django.core.exceptions import PermissionDenied, BadRequest
-from django.db.models import Max
+from django.db.models import Max, Q
 from django.http import HttpRequest, HttpResponse, Http404, JsonResponse
 from django.utils import timezone
 
@@ -24,8 +24,6 @@ def api(request: HttpRequest) -> HttpResponse:
             return api_create_project(request)
         case "edit-project":
             return api_edit_project(request)
-        case "edit-project-ranks":
-            return api_edit_project_ranks(request)
         case "archive-project":
             return api_archive_project(request)
         case "unarchive-project":
@@ -90,32 +88,31 @@ def api(request: HttpRequest) -> HttpResponse:
 
 @permission_required("orgapy.view_project")
 def api_list_projects(request: HttpRequest) -> JsonResponse:
-    show_archived = request.GET.get("archived", "0") == "1"
+    note_filter = request.GET.get("note")
+    status_filter = request.GET.get("status")
+    project_filter = request.GET.get("project")
     projects = []
-    for project in Project.objects.filter(user=request.user):
-        if project.archived and not show_archived:
-            continue
-        progress = None
-        if project.progress_min is not None and project.progress_max is not None and project.progress_current is not None:
-            progress = {
-                "min": project.progress_min,
-                "max": project.progress_max,
-                "current": project.progress_current,
-            }
-        projects.append({
-            "id": project.id,
-            "creation": project.date_creation.timestamp(),
-            "modification": project.date_modification.timestamp(),
-            "title": project.title,
-            "category": project.category,
-            "limitDate": project.limit_date,
-            "progress": progress,
-            "description": project.description if project.description else None,
-            "checklist": project.checklist if project.checklist else None,
-            "rank": project.rank,
-            "note": None if project.note is None else project.note.id,
-            "archived": project.archived,
-        })
+    query = Project.objects.filter(user=request.user)
+    if project_filter is not None:
+        try:
+            query = query.filter(id=int(project_filter))
+        except:
+            raise BadRequest()
+    if note_filter is not None:
+        try:
+            query = query.filter(note__id=int(note_filter))
+        except ValueError:
+            pass
+    if status_filter is not None:
+        nodes = None
+        for status in status_filter.split(","):
+            if nodes is None:
+                nodes = Q(status=status)
+            else:
+                nodes |= Q(status=status)
+        query = query.filter(nodes)
+    for project in query:
+        projects.append(project.to_json_dict())
     return JsonResponse({"projects": projects})
 
 
@@ -128,112 +125,13 @@ def api_create_project(request: HttpRequest) -> JsonResponse:
         max_rank = 1
     project = Project.objects.create(
         user=request.user,
-        title="New Project",
-        category="general",
         rank=int(max_rank) + 1
     )
-    return JsonResponse({"success": True, "project": {
-        "id": project.id,
-        "title": project.title,
-        "category": project.category,
-        "limitDate": None,
-        "progress": None,
-        "description": None,
-        "checklist": None,
-        "creation": project.date_creation.timestamp(),
-        "modification": project.date_modification.timestamp(),
-        "rank": project.rank,
-        "note": None,
-        "archived": False,
-    }})
+    # TODO: handle title & note
+    return JsonResponse({"success": True, "project": project.to_json_dict()})
 
 
-@permission_required("orgapy.change_project")
-def api_edit_project(request: HttpRequest) -> JsonResponse:
-    if request.method != "POST":
-        raise BadRequest()
-    project_id = request.POST.get("project_id")
-    project_data = request.POST.get("project_data")
-    if project_id is None or project_data is None:
-        raise BadRequest()
-    try:
-        project_id = int(project_id)
-        project_data = json.loads(project_data)
-    except:
-        raise BadRequest()
-    if not Project.objects.filter(id=project_id, user=request.user).exists():
-        raise Http404()
-    project = Project.objects.get(id=project_id, user=request.user)
-    if project.date_modification.timestamp() > project_data["modification"]:
-        return JsonResponse({"success": False, "reason": "Project has newer modifications"})
-    project.title = project_data["title"]
-    project.category = project_data["category"]
-    project.rank = float(project_data["rank"])
-    if project_data["limitDate"] is not None:
-        project.limit_date = datetime.datetime.strptime(project_data["limitDate"], "%Y-%m-%d").date()
-    else:
-        project.limit_date = None
-    if project_data["progress"] is not None:
-        project.progress_min = project_data["progress"]["min"]
-        project.progress_max = project_data["progress"]["max"]
-        project.progress_current = project_data["progress"]["current"]
-    else:
-        project.progress_min = None
-        project.progress_max = None
-        project.progress_current = None
-    if project_data["description"] is not None:
-        project.description = project_data["description"]
-    else:
-        project.description = None
-    if isinstance(request.user, AnonymousUser):
-        raise PermissionDenied()
-    compare_checklists(request.user, project.title, project.checklist, project_data["checklist"])
-    if project_data["checklist"] is not None:
-        project.checklist = project_data["checklist"]
-    else:
-        project.checklist = None
-
-    note = None
-    if project_data["note"] is not None:
-        try:
-            note = Note.objects.get(user=request.user, id=int(project_data["note"]))
-        except Note.DoesNotExist:
-            pass
-    project.note = note # type: ignore
-
-    project.save()
-    return JsonResponse({
-        "success": True,
-        "modification": project.date_modification.timestamp()
-    })
-
-
-@permission_required("orgapy.change_project")
-def api_edit_project_ranks(request: HttpRequest) -> JsonResponse:
-    if request.method != "POST":
-        raise BadRequest()
-    ranks_data = request.POST.get("ranks")
-    if ranks_data is None:
-        raise BadRequest()
-    try:
-        ranks_data = json.loads(ranks_data)
-    except:
-        raise BadRequest()
-    projects, ranks = [], []
-    for project_id, rank in ranks_data.items():
-        if not Project.objects.filter(id=int(project_id), user=request.user).exists():
-            raise Http404()
-        project = Project.objects.get(id=int(project_id), user=request.user)
-        projects.append(project)
-        ranks.append(float(rank))
-    for project, rank in zip(projects, ranks):
-        project.rank = rank
-        project.save()
-    return JsonResponse({"success": True})
-
-
-@permission_required("orgapy.change_project")
-def api_archive_project(request: HttpRequest) -> JsonResponse:
+def get_project_from_post(request: HttpRequest) -> Project:
     if request.method != "POST":
         raise BadRequest()
     project_id = request.POST.get("project_id")
@@ -243,9 +141,51 @@ def api_archive_project(request: HttpRequest) -> JsonResponse:
         project_id = int(project_id)
     except:
         raise BadRequest()
-    if not Project.objects.filter(id=project_id, user=request.user).exists():
+    try:
+        project = Project.objects.get(id=project_id, user=request.user)
+    except Project.DoesNotExist:
         raise Http404()
-    project = Project.objects.get(id=project_id, user=request.user)
+    return project
+
+
+@permission_required("orgapy.change_project")
+def api_edit_project(request: HttpRequest) -> JsonResponse:
+    project = get_project_from_post(request)
+    project_data = request.POST.get("project_data")
+    if project_data is None:
+        raise BadRequest()
+    try:
+        project_data = json.loads(project_data)
+    except:
+        raise BadRequest()
+    if project.date_modification.timestamp() > project_data["modification"]:
+        return JsonResponse({"success": False, "reason": "Project has newer modifications"})
+    project.title = project_data["title"]
+    if isinstance(request.user, AnonymousUser):
+        raise PermissionDenied()
+    compare_checklists(request.user, project.reference, project.checklist, project_data["checklist"])
+    if project_data["checklist"] is not None:
+        project.checklist = project_data["checklist"]
+    else:
+        project.checklist = None
+    project.status = project_data["status"]
+    note = None
+    if project_data["note"] is not None:
+        try:
+            note = Note.objects.get(user=request.user, id=int(project_data["note"]["id"]))
+        except Note.DoesNotExist:
+            pass
+    project.note = note # type: ignore
+    project.save()
+    return JsonResponse({
+        "success": True,
+        "modification": project.date_modification.timestamp()
+    })
+
+
+@permission_required("orgapy.change_project")
+def api_archive_project(request: HttpRequest) -> JsonResponse:
+    project = get_project_from_post(request)
     project.archived = True
     project.save()
     return JsonResponse({
@@ -256,18 +196,7 @@ def api_archive_project(request: HttpRequest) -> JsonResponse:
 
 @permission_required("orgapy.change_project")
 def api_unarchive_project(request: HttpRequest) -> JsonResponse:
-    if request.method != "POST":
-        raise BadRequest()
-    project_id = request.POST.get("project_id")
-    if project_id is None:
-        raise BadRequest()
-    try:
-        project_id = int(project_id)
-    except:
-        raise BadRequest()
-    if not Project.objects.filter(id=project_id, user=request.user).exists():
-        raise Http404()
-    project = Project.objects.get(id=project_id, user=request.user)
+    project = get_project_from_post(request)
     project.archived = False
     project.save()
     return JsonResponse({
@@ -278,18 +207,7 @@ def api_unarchive_project(request: HttpRequest) -> JsonResponse:
 
 @permission_required("orgapy.delete_project")
 def api_delete_project(request: HttpRequest) -> JsonResponse:
-    if request.method != "POST":
-        raise BadRequest()
-    project_id = request.POST.get("project_id")
-    if project_id is None:
-        raise BadRequest()
-    try:
-        project_id = int(project_id)
-    except:
-        raise BadRequest()
-    if not Project.objects.filter(id=project_id, user=request.user).exists():
-        raise Http404()
-    project = Project.objects.get(id=project_id, user=request.user)
+    project = get_project_from_post(request)
     project.delete()
     return JsonResponse({"success": True})
 
