@@ -10,7 +10,7 @@ from django.http import HttpRequest, HttpResponse, Http404
 from django.shortcuts import redirect, render
 from django.utils.text import slugify
 
-from ..models import Category, Note, Sheet, Map, ProgressCounter, ProgressLog, Calendar, PushSubscription
+from ..models import Category, Note, Sheet, Map, ProgressCounter, ProgressLog, Calendar, PushSubscription, Project
 from .utils import ConflictError, find_user_object, pretty_paginator, save_document_core, get_or_create_settings, toggle_object_attribute, view_documents_single, view_documents_mixed
 
 
@@ -28,6 +28,9 @@ def view_about(request: HttpRequest) -> HttpResponse:
     return render(request, "orgapy/about.html", {})
 
 
+# PROJECTS #####################################################################
+
+
 @permission_required("orgapy.view_project")
 def view_projects(request: HttpRequest) -> HttpResponse:
     if isinstance(request.user, AnonymousUser):
@@ -38,6 +41,57 @@ def view_projects(request: HttpRequest) -> HttpResponse:
     })
 
 
+@permission_required("orgapy.view_project")
+def view_projects_archive(request: HttpRequest) -> HttpResponse:
+    note_filter = request.GET.get("note")
+    query = Project.objects.filter(user=request.user, status=Project.ARCHIVED)
+    if note_filter is not None:
+        try:
+            query = query.filter(note__id=int(note_filter))
+        except:
+            raise BadRequest()
+    query = query.order_by("-date_modification")
+    page_size = 24
+    paginator = Paginator(query, page_size)
+    page = request.GET.get("page")
+    projects = paginator.get_page(page)
+    return render(request, "orgapy/projects_archive.html", {
+        "projects": projects,
+        "paginator": pretty_paginator(projects),
+        "active": "projects",
+    })
+
+
+@permission_required("orgapy.view_project")
+def view_project(request: HttpRequest, object_id: str) -> HttpResponse:
+    if isinstance(request.user, AnonymousUser):
+        raise PermissionDenied()
+    project = find_user_object(Project, "id", object_id, request.user)
+    return render(request, "orgapy/project.html", {
+        "project": project,
+        "active": "projects",
+    })
+
+
+@permission_required("orgapy.change_project")
+def view_unarchive_project(request: HttpRequest, object_id: str) -> HttpResponse:
+    project = find_user_object(Project, "id", object_id, request.user)
+    project.status = Project.ACTIVE
+    project.save()
+    if "next" in request.GET:
+        return redirect(request.GET["next"])
+    return redirect(project.get_absolute_url())
+
+
+@permission_required("orgapy.delete_project")
+def view_delete_project(request: HttpRequest, object_id: str) -> HttpResponse:
+    project = find_user_object(Project, "id", object_id, request.user)
+    project.delete()
+    if "next" in request.GET:
+        return redirect(request.GET["next"])
+    return redirect("orgapy:projects")
+
+
 # OBJECTS AND CATEGORIES #######################################################
 
 
@@ -269,6 +323,23 @@ def view_note(request: HttpRequest, object_id: str) -> HttpResponse:
     })
 
 
+def view_note_standalone(request: HttpRequest, object_id: str) -> HttpResponse:
+    note = find_user_object(Note, ["id", "nonce"], object_id)
+    has_permission = False
+    if request.user is not None and note.user == request.user and request.user.has_perm("orgapy.view_note"):
+        has_permission = True
+    elif note.public and isinstance(object_id, str) and len(object_id) == 12:
+        has_permission = True
+    if not has_permission:
+        raise PermissionDenied()
+    response = render(request, "orgapy/note_standalone.html", {
+        "note": note,
+        "readonly": True,
+    })
+    response["X-Frame-Options"] = "SAMEORIGIN"
+    return response
+
+
 @permission_required("orgapy.change_note")
 def view_edit_note(request: HttpRequest, object_id: str) -> HttpResponse:
     note = find_user_object(Note, "id", object_id, request.user)
