@@ -11,10 +11,10 @@ from django.http import HttpRequest, Http404, HttpResponse
 from django.shortcuts import redirect, render
 from django.utils import timezone
 
-from ..models import Settings, Category, Note, Sheet, Map, ProgressLog, Document, PushSubscription
+from ..models import Settings, Category, Note, Sheet, Map, ProgressLog, Document, PushSubscription, Project
 
 
-UserObject = TypeVar("UserObject", Category, Note, Sheet, Map, ProgressLog, PushSubscription)
+UserObject = TypeVar("UserObject", Category, Note, Sheet, Map, ProgressLog, PushSubscription, Project)
 DocumentT = TypeVar("DocumentT", Note, Sheet, Map)
 LoggedUser = AbstractBaseUser
 
@@ -58,6 +58,12 @@ def find_user_object(
         allow_deleted: bool = False,
         ) -> UserObject:
     fields = [key] if isinstance(key, str) else key
+    try:
+        int(value)
+    except ValueError:
+        fields.remove("id")
+    if not fields:
+        raise BadRequest()
     nodes = Q(**{fields[0]: value})
     for field in fields[1:]:
         nodes |= Q(**{field: value})
@@ -79,7 +85,7 @@ def get_checked_items(checklist: str) -> list[str]:
     return re.findall(r"\[x\] (.*)", checklist)
 
 
-def compare_checklists(user: LoggedUser, title: str, before: str | None, after: str | None):
+def compare_checklists(user: LoggedUser, reference: str, before: str | None, after: str | None):
     if before is None or after is None:
         return
     checked_before = set(get_checked_items(before))
@@ -88,7 +94,7 @@ def compare_checklists(user: LoggedUser, title: str, before: str | None, after:
         ProgressLog.objects.create(
             user=user,
             type=ProgressLog.PROJECT_CHECKLIST_ITEM_CHECKED,
-            description=f"{title} - {item}"
+            description=f"{reference} - {item}"
         )
 
 
